<?php

namespace Drupal\ics_link_field\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystem;
use Drupal\Core\Session\AccountProxy;
use Drupal\Component\Utility\Html;
use Eluceo\iCal\Component\Calendar;
use Eluceo\iCal\Component\Event;

/**
 * Class ServiceController
 *
 * @package Drupal\zhi_event_addtocal\Controller
 */
class ServiceController extends ControllerBase {

  /**
   * @var \Drupal\Core\File\FileSystem $file
   */
  protected $file;



  /** @var \Drupal\Core\Utility\Token $token */
  protected $token;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Session\AccountProxy $currentUser
   */
  protected $currentUser;

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack $request
   */
  public $request;


  /**
   * ServiceController constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\Core\Session\AccountProxy $currentuser
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   * @param \Drupal\Core\File\FileSystem $fileStorage
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, AccountProxy $currentuser, RequestStack $request, FileSystem $fileStorage, Token $token) {
    $this->entityTypeManager = $entityTypeManager;
    $this->currentUser = $currentuser;
    $this->request = $request;
    $this->file = $fileStorage;
    $this->token = $token;
  }

  /**
   * Create dependency injection.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('request_stack'),
      $container->get('file_system'),
      $container->get('token')
    );
  }

  /**
   * ICS file download response.
   *
   * @param string $field_name
   *   The field_name which stores ics data.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity this ics will be generated from.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Exception
   */
  public function icsDownload($field_name, EntityInterface $entity) {

    if (empty($entity)) {
      throw new NotFoundHttpException();
    }

    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->getType();

    $field_config_name = implode(".", [$entity_type, $bundle, $field_name]);

    $field_config = $this->entityTypeManager->getStorage('field_config')->load($field_config_name);
    $field_settings = $field_config->getSettings();

    //@TODO: allow more token types
    $data = [$entity_type => $entity];

    foreach ($field_settings as &$setting_value) {
      $setting_value =  $this->token->replace($setting_value, $data);
    }

    // @TODO: validate inputs (HTML Date for "date" fields / HTML Time for "time" fields)
    if (empty($field_settings['start_date'])) {
      throw new NotFoundHttpException();
    }

    if (empty($field_settings['end_date'])) {
      $field_settings['end_date'] = $field_settings['start_date'];
    }

    $start_date = new \DateTime($field_settings['start_date']);
    if($start_date === FALSE) {
      throw new NotFoundHttpException();
    }

    $end_date = new \DateTime($field_settings['end_date']);
    if($end_date === FALSE) {
      throw new NotFoundHttpException();
    }

    $host = $this->request->getCurrentRequest()->getHost();
    $vCalendar = new Calendar($host);

    if(empty($field_settings['start_time']) || empty($field_settings['end_time'])) {
      $event = $this->createEvent($field_settings, $start_date, $end_date, TRUE);
      $vCalendar->addComponent($event);
    }
    else {
      // Create multiple events, one for each day.
      // Even if it's a single day event, at least one event will be created.
      do {
        $daily_start = new \DateTime($start_date->format('Y-m-d') . " " . $field_settings['start_time']);
        $daily_end = new \DateTime($start_date->format('Y-m-d') . " " . $field_settings['end_time']);

        $event = $this->createEvent($field_settings, $daily_start, $daily_end);
        $vCalendar->addComponent($event);

        $start_date->modify("+1 day");
      } while ($start_date < $end_date);
    }

    $filename = 'event-' . strtolower(Html::cleanCssIdentifier($field_settings['title'])) . '.ics';
    $content = $vCalendar->render();

    $response = new Response($content);

    // Create the disposition of the file
    $disposition = $response->headers->makeDisposition(
      ResponseHeaderBag::DISPOSITION_ATTACHMENT,
      $filename
    );

    // Set the content disposition
    $response->headers->set('Content-Disposition', $disposition);
    $response->headers->set('Content-Type', ['text/calendar', 'charset=utf-8']);

    // @TODO: add caching per entity

    // Dispatch request
    return $response;
  }

  function createEvent($field_settings, \DateTime $dtStart, \DateTime $dtEnd, bool $noTime = FALSE) {
    $event = new Event();
    $event->setSummary($field_settings['title']);
    $event->setDescription($field_settings['description']);
    $event->setLocation($field_settings['location']);
    $event->setNoTime($noTime);
    $event->setDtStart($dtStart);
    $event->setDtEnd($dtEnd);
    return $event;
  }
}
